/* YOODA : Yet another Object Oriented Database
   Copyright (C) 1994  Eric Abecassis

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Library General Public
License as published by the Free Software Foundation; either
version 2 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the Free
Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA. */


/*****************************************************************************
******************************************************************************
**                                                                            
**        Y     Y   OOOO    OOOO   DDDDD     AA
**         Y   Y   O    O  O    O  D    D   A  A
**          Y Y    O    O  O    O  D    D  A    A 
**           Y     O    O  O    O  D    D  AAAAAA
**           Y     O    O  O    O  D    D  A    A
**           Y      OOOO    OOOO   DDDDD   A    A
**
******************************************************************************
******************************************************************************/
/*****************************************************************************
*
*  MODULE :             String management
*  SCOPE :              String management
*  FILE :               stringx.cc
*  AUTHORS :            E. Abecassis
*  CREATION DATE :      August 1, 1993
*  COPYRIGHTS :        (C) E. Abecassis, 1993
*  HISTORY :
*
******************************************************************************/

#ifdef __GNUG__
#pragma implementation
#endif

/* IMPORTS *******************************************************************/

#ifndef __stringx_HXX
#include "stringx.hxx"
#endif

#ifndef __yooexcept_HXX
#include "yooexcept.hxx"
#endif

/*****************************************************************************/

/*= EXTERNAL FUNCTION =======================================================*/
int strcmp_special(char * s1,char * s2)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  while (*s1) {
    if (*(s1++) != *(s2++)) {
      return 0;
    }
  }
  if (*s2) {
    return 0;
  } else {
    return 1;
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
StringX::StringX()
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  if (YOO_LOAD_TIME == FALSE)  { 
    ref = NULL ;
  }  
}


/*= EXTERNAL FUNCTION =======================================================*/
StringX::StringX(const StringX& s)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  ref = NULL ;
  this->assign(s) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
StringX::StringX(char* s)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  ref = NULL ;
  this->assign(s) ;
}


/*= EXTERNAL FUNCTION =======================================================*/
StringX& StringX::operator+=(char* s)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  int oldsize = this->ref->objsize() ;
  int newsize = oldsize + strlen(s) ;
  int oldpage = yoo_pageSize(oldsize + PAGE_HEADER_SIZE) ;
  int newpage = yoo_pageSize(newsize + PAGE_HEADER_SIZE) ;
  
  if ( (oldpage!=newpage) ||
       ( (oldpage == 1) && (yoo_slotSize(oldsize)!=yoo_slotSize(newsize)) ) ) {
    this->ref = (_StringX*)this->ref->changeSize(yoo_slotSize(newsize)) ;
  }

  strcat(this->data(), s) ;  
  return *this ;
}


/*= EXTERNAL FUNCTION =======================================================*/
void StringX::alloc(int size)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  PageHdlMem* pd = yoo_getpage(this) ;
  Cluster* clu =  pd != NULL ? pd->page()->getCluster() : YOO_TMP_CLU ;
  this->ref = (_StringX*)ObjectX::operator new(sizeof(_StringX) + size,
					       clu, UNTYPED,
					       (pd==NULL) ? NULL : pd->page());
}


/*= EXTERNAL FUNCTION =======================================================*/
void StringX::assign(char* s1)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->clear() ;
  if (s1 != NULL) {
    int size = strlen(s1) + 1 ;
    this->alloc(size) ;
    strcpy(this->data(),s1);
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
void StringX::assign(char* s1, char* s2)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->clear() ;
  if (s1 != NULL) {
    int size = strlen(s1) + strlen(s2) +1 ;
    this->alloc(size) ;
    strcpy(this->data(),s1);
    strcat(this->data(),s2);
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
void StringX::assign(char* s1, char* s2, char* s3)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->clear() ;
  if (s1 != NULL) {
    int size = strlen(s1) + strlen(s2) + strlen(s3) +1 ;
    this->alloc(size) ;
    strcpy(this->data(),s1);
    strcat(this->data(),s2);
    strcat(this->data(),s3);
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
void StringX::assign(char* s1, char* s2, char* s3, char* s4)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->clear() ;
  if (s1 != NULL) {
    int size = strlen(s1) + strlen(s2) + strlen(s3) + strlen(s4) + 1 ;
    this->alloc(size) ;
    strcpy(this->data(),s1);
    strcat(this->data(),s2);
    strcat(this->data(),s3);
    strcat(this->data(),s4);
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
void StringX::assign(char* s1, char* s2, char* s3, char* s4, char* s5)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  this->clear() ;
  if (s1 != NULL) {
    int size=strlen(s1) + strlen(s2) + strlen(s3) + strlen(s4) + strlen(s5) +1;
    this->alloc(size) ;
    strcpy(this->data(),s1);
    strcat(this->data(),s2);
    strcat(this->data(),s3);
    strcat(this->data(),s4);
    strcat(this->data(),s5);
  }
}


/*= EXTERNAL FUNCTION =======================================================*/
char& StringX::operator[](int i)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
#ifdef DEBUG
  int size = this->ref->objsize() - sizeof(ObjectX) ;
  if ((i<0) || (i>=size)) {
    call_throw(YOO_ERR_STR, "String Indice out of bounds") ;
  }
#endif

  return (this->data())[i] ;
}


/*= EXTERNAL FUNCTION =======================================================*/
StringX StringX::operator()(int index, int count)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  int size = strlen(this->data()) ;
  if ( (index < 0) || ((index+count) > size) ) {
    call_throw(YOO_ERR_STR, "String Indice out of bounds") ;
  }
  if (count > 0) {
    StringX s ;
    s.alloc(count+1) ;
    s[0] = 0 ;
    strncpy(s.data(),this->data()+index,count);
    s[count] = '\0';
    return s ;
  } else {
    //une chaine vide
    StringX s(NULL) ;
    return s ;
  }
} 


/*= EXTERNAL FUNCTION =======================================================*/
StringX operator+(const StringX& s1,const char* s2)
/*=
[DESCRIPTION :]
  (IN)  
  (OUT) 
[DIAGNOSTIC :]
[CALL :]
=*/
{
  StringX s;
  s.alloc(strlen(s1) + strlen(s2)) ;
  s[0] = 0 ;
  strcpy(s.data(),s1.data());
  strcat(s.data(),s2);
  return s ;
}


